/**
 * eobjects.org AnalyzerBeans
 * Copyright (C) 2010 eobjects.org
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */
package org.eobjects.analyzer.job;

import java.util.List;

import org.eobjects.analyzer.job.builder.LazyFilterOutcome;

import org.eobjects.metamodel.util.BaseObject;

/**
 * Provides hashCode, equals and toString implementations for FilterOutcome,
 * making them comparable across different implementations.
 * 
 * Specifically this has been designed to make it possible to use the
 * equals(...) method with both ImmutableFilterOutcome and LazyFilterOutcome
 * instances.
 * 
 * @see ImmutableFilterOutcome
 * @see LazyFilterOutcome
 * 
 * @author Kasper Sørensen
 * 
 */
public abstract class AbstractFilterOutcome extends BaseObject implements FilterOutcome {
	
	private static final long serialVersionUID = 1L;

	@Override
	protected final void decorateIdentity(List<Object> identifiers) {
		identifiers.add(getCategory());
		identifiers.add(getSourceJob());
	}

	@Override
	protected final boolean classEquals(BaseObject obj) {
		// works with all subtypes
		return obj instanceof FilterOutcome;
	}

	@Override
	public String toString() {
		return "FilterOutcome[category=" + getCategory() + "]";
	}

	@Override
	public final boolean satisfiesRequirement(Outcome requirement) {
		return equals(requirement);
	}
}
